/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.remote;

import java.rmi.RemoteException;
import java.security.cert.X509Certificate;
import java.util.List;

import kgrid.client.gui.Settings;
import kgrid.client.gui.View;
import kgrid.das.stubs.DASFactory.CreateResource;
import kgrid.das.stubs.DASFactory.CreateResourceResponse;
import kgrid.das.stubs.DASFactory.DASFactoryPortType;
import kgrid.das.stubs.DASFactory.service.DASFactoryServiceAddressingLocator;
import kgrid.das.stubs.DASService.ArrayOfString;
import kgrid.das.stubs.DASService.DASPortType;
import kgrid.das.stubs.DASService.service.DASServiceAddressingLocator;

import org.apache.axis.message.addressing.Address;
import org.apache.axis.message.addressing.EndpointReferenceType;
import org.globus.axis.util.Util;
import org.globus.delegation.DelegationUtil;
import org.globus.gsi.GSIConstants;
import org.globus.gsi.GlobusCredential;
import org.globus.wsrf.NotificationConsumerManager;
import org.globus.wsrf.NotifyCallback;
import org.globus.wsrf.WSNConstants;
import org.globus.wsrf.core.notification.ResourcePropertyValueChangeNotificationElementType;
import org.globus.wsrf.impl.security.authorization.Authorization;
import org.globus.wsrf.impl.security.authorization.HostAuthorization;
import org.globus.wsrf.impl.security.descriptor.ClientSecurityDescriptor;
import org.globus.wsrf.utils.AddressingUtils;
import org.jdom.Element;
import org.jdom.output.XMLOutputter;
import org.oasis.wsn.Subscribe;
import org.oasis.wsn.TopicExpressionType;
import org.oasis.wsrf.properties.ResourcePropertyValueChangeNotificationType;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class DASTask implements NotifyCallback, Consts {

	private DASPortType dasPT;
	private String factoryURI;
	private ArrayOfString queryString;
	private String result;
	private View view;

	static {
		Util.registerTransport();
	}

	public DASTask(Element queryElement, View view) throws Exception {

		String message = "Generating query...";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);

		XMLOutputter outputter = new XMLOutputter();
		String xml = outputter.outputString(queryElement);
		System.out.println("xml = " + xml);

		this.queryString = getQueryFromXML(xml);
		this.view = view;

		for (String s : queryString.getX()) {
			System.out.println(s);
		}

		message = "Initializing the DAS service invocation...";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);

		DASFactoryServiceAddressingLocator factoryLocator = new DASFactoryServiceAddressingLocator();
		DASServiceAddressingLocator instanceLocator = new DASServiceAddressingLocator();

		factoryURI = "https://" + Settings.getDASAddress() + ":"
				+ Settings.getDASPort() + "/" + DAS_FACTORY;

		EndpointReferenceType factoryEPR, instanceEPR;
		DASFactoryPortType dasFactory;
		message = "Getting DAS Factory Port Type...";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);

		// 1: get Factory Port Type
		factoryEPR = new EndpointReferenceType();
		factoryEPR.setAddress(new Address(factoryURI));
		dasFactory = factoryLocator.getDASFactoryPortTypePort(factoryEPR);

		// 2: Create resource and get endpoint reference of WS-Resource.
		// This resource is our "instance".
		message = "Creating resource and getting endpoint reference of WS-Resource";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);

		CreateResourceResponse createResponse = dasFactory
				.createResource(new CreateResource());
		instanceEPR = createResponse.getEndpointReference();

		// 3: ?
		dasPT = instanceLocator.getDASPortTypePort(instanceEPR);
		subscribeToDASModelTopic(dasPT);

		EndpointReferenceType epr = delegateCredential();

		kgrid.client.gui.Utils.print("epr = " + epr.toString());
	}

	/**
	 * Invokes RAEMS' getQueryFromXML to obtain the query string and returns a
	 * DAS ArrayOfString from the RAEMS ArrayOfString.
	 */
	private static ArrayOfString getQueryFromXML(String xml)
			throws RemoteException {
		return new ArrayOfString(Utils.getRAEMSQueryAOS(xml).getX());
	}

	public void invoke() {
		String message = "Invoking the DAS service...";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);
		try {
			dasPT.searchResource(queryString);
			message = "DAS service invoked, waiting for the results...";
			view.setStatus(message);
			kgrid.client.gui.Utils.print(message);
		} catch (RemoteException e) {
			e.printStackTrace();
		}
	}

	public synchronized String getResult() {
		try {
			wait();
		} catch (Exception e) {
			e.printStackTrace();
		}
		String message = "Results received";
		view.setStatus(message);
		kgrid.client.gui.Utils.print(message);
		return result;
	}

	/**
	 * Contacts the Delegation Factory Service and gets an EPR for the
	 * Delegation Resource that contains the delegated credential.
	 */
	private static EndpointReferenceType delegateCredential() throws Exception {
		String host = Settings.getDASAddress();
		int port = Settings.getDASPort();

		ClientSecurityDescriptor desc = getClientSecDesc();
		GlobusCredential credential = GlobusCredential.getDefaultCredential();
		String factoryUrl = ("https://" + host + ":" + port + "/" + DELEGATION_FACTORY);
		int terminationTime = 60;
		int lifetime = terminationTime * 60; // lifetime in seconds
		EndpointReferenceType delegEpr = AddressingUtils
				.createEndpointReference(factoryUrl, null);
		X509Certificate[] certsToDelegateOn = DelegationUtil
				.getCertificateChainRP(delegEpr, desc);
		X509Certificate certToSign = certsToDelegateOn[0];
		return DelegationUtil.delegate(factoryUrl, credential, certToSign,
				lifetime, false, desc);
	}

	private static ClientSecurityDescriptor getClientSecDesc() {
		ClientSecurityDescriptor desc = new ClientSecurityDescriptor();
		Authorization authzVal = HostAuthorization.getInstance();
		Object authVal = GSIConstants.SIGNATURE;
		desc.setGSITransport((Integer) authVal);
		Util.registerTransport();
		desc.setAuthz(authzVal);
		return desc;
	}

	/**
	 * The deliver function of the interface {@link NotifyCallback} will be
	 * invoked whenever a notification for that consumer arrives. The message
	 * parameter will either be of {@link Element} type if the message type was
	 * unknown or some object mapped to the type of the message.
	 */
	public synchronized void deliver(List topicPath,
			EndpointReferenceType producer, Object message) {
		synchronized (this) {
			kgrid.client.gui.Utils
					.print("Notification delivered from DAS service");
			ResourcePropertyValueChangeNotificationElementType notifElem;
			ResourcePropertyValueChangeNotificationType notif;

			notifElem = (ResourcePropertyValueChangeNotificationElementType) message;
			notif = notifElem.getResourcePropertyValueChangeNotification();

			if (notif != null) {
				// result = notif.getOldValue().get_any()[0].getValue();
				result = notif.getNewValue().get_any()[0].getValue();
			}
			Utils.print("Notification received", producer.getAddress());
			synchronized (this) {
				notify();
			}
		}
	}

	/**
	 * Subscribe the specified listener as consumer of notification 'KDSModel'
	 * topic of the specified KDS Service
	 * 
	 * @param kds
	 *            KDSPortType of the KDS Service whose the topic is a resource
	 *            property
	 */
	public void subscribeToDASModelTopic(DASPortType dasPT) throws Exception {

		// 1: set up the notification consumer
		// (the listener will be set up as a notification consumer)
		NotificationConsumerManager consumer = null;

		// 2: set the 'listener' as a 'notification consumer'
		consumer = NotificationConsumerManager.getInstance();
		consumer.startListening();
		EndpointReferenceType consumerEPR = consumer
				.createNotificationConsumer(this);

		// 3: create (and opportunetely set) the remote subscribe request
		// Create the request to the remote Subscribe() call
		Subscribe request = new Subscribe();
		// Must the notification be delivered using the Notify
		// operation?
		request.setUseNotify(Boolean.TRUE);
		// Indicate what the consumer client's EPR is
		request.setConsumerReference(consumerEPR);

		// 4: create the topic to be subscribed to
		// The TopicExpression specifies what topic we want to
		// subscribe to
		TopicExpressionType topicExpression = new TopicExpressionType();
		topicExpression.setDialect(WSNConstants.SIMPLE_TOPIC_DIALECT);
		topicExpression.setValue(DAS_QNAME);
		request.setTopicExpression(topicExpression);

		// 6: send the subscribe request to the notification producer (EPMS)

		dasPT.subscribe(request);
	}

}
