/*
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU General Public License as published by the Free Software
 * Foundation; either version 2 of the License, or (at your option) any later
 * version.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE. See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU General Public License along with
 * this program; if not, write to the Free Software Foundation, Inc., 675 Mass
 * Ave, Cambridge, MA 02139, USA.
 * 
 * =============================================================================
 * Grid Computing Laboratory, Dept. of Electronics, Computer Science and
 * Systems, University of Calabria, Italy. http://grid.deis.unical.it/
 * =============================================================================
 */

package kgrid.client.workflow;

import java.util.LinkedHashMap;
import java.util.Map;

import org.jdesktop.application.ResourceMap;
import org.jdom.Element;
import org.jdom.JDOMException;

/**
 * @author Marco Lackovic <mlackovic@deis.unical.it>
 */
public class ToolNode extends WorkflowNode {

	private static final long serialVersionUID = 1L;
	private static final String NAME = "Tool";
	private static final int TYPE = TOOL_TYPE;
	public static final int NUM_DEFAULT_PROPERTIES = 6;

	private String matchedKDSURL = null;

	private LinkedHashMap<String, String> args = new LinkedHashMap<String, String>();

	public ToolNode(int id, int x, int y, ResourceMap resourceMap, Workflow w) {
		super(id, NAME, x, y, resourceMap, w);
		properties.put(PKDS, "");
		properties.put(PNAME, "");
		properties.put(PKDATA, "");
		properties.put(PKSOFT, "");
		properties.put(PHOST, "");
		properties.put(PARGS, args);
	}

	@Override
	public void clearSearchParameters() {
		properties.put(PNAME, "");
		properties.put(PKDATA, "");
		properties.put(PKSOFT, "");
		properties.put(PHOST, "");
	}

	@Override
	public String getKDSURL() {
		return matchedKDSURL;
	}

	@Override
	protected void setKDSURL(String s) {
		matchedKDSURL = s;
	}

	public void setProperties(Object description) throws JDOMException {
		if (description != null) {
			Element tool = (Element) description;
			if (tool.getName().equals("DataMiningSoftware")) {
				String href = tool.getAttributeValue("href");
				if (href != null && !href.isEmpty()
						&& !href.equals(KDSURL_PREFIX)) {
					properties.put(PKDS, href);
					matchedKDSURL = href;
				} else {
					// String title = tool.getAttributeValue("title");
					String name = tool.getAttributeValue("name");
					if (name != null) {
						properties.put(PNAME, name);
					}
				}
				for (Object toolChild : tool.getChildren()) {
					Element toolElement = (Element) toolChild;
					if (toolElement.getName().equals("Description")) {
						for (Object descriptionChild : toolElement
								.getChildren()) {
							Element descriptionElement = (Element) descriptionChild;
							if (descriptionElement.getName().equals(
									"KindOfData")) {
								properties.put(PKDATA,
										descriptionElement.getText());
							} else {
								throw new JDOMException("Tag <"
										+ descriptionElement.getName()
										+ "> unknown");
							}
						}
					} else if (toolElement.getName().equals("Usage")) {
						for (Object usageChild : toolElement.getChildren()) {
							Element usageElement = (Element) usageChild;
							if (usageElement.getName().equals("KindOfSoftware")) {
								properties.put(PKSOFT, usageElement.getText());
							} else if (usageElement.getName()
									.equals("HostName")) {
								properties.put(PHOST, usageElement.getText());
							} else if (usageElement.getName().equals("Args")) {
								addArgs(usageElement);
							} else {
								throw new JDOMException("Tag <"
										+ usageElement.getName() + "> unknown");
							}
						}
					} else {
						throw new JDOMException("Tag <" + toolElement.getName()
								+ "> unknown");
					}
				}
			} else {
				throw new JDOMException("Tag <" + tool.getName() + "> unknown");
			}
			resetStatus();
		}

	}

	@Override
	public String toString() {
		String kdsUrl = (String) properties.get(PKDS);
		if (!kdsUrl.isEmpty() && !kdsUrl.equals(KDSURL_PREFIX)) {
			kdsUrl = kdsUrl.substring(kdsUrl.lastIndexOf("/") + 1);
			int ext = kdsUrl.lastIndexOf(".");
			if (ext > 0) {
				return kdsUrl.substring(0, ext);
			}
			return kdsUrl;
		}
		return (String) properties.get(PNAME);
	}

	private void addArgs(Element args) throws JDOMException {

		for (Object argsChild : args.getChildren()) {
			Element arg = (Element) argsChild;
			if (arg.getName().equals("Arg")) {
				this.args.put(arg.getAttributeValue("name"), arg.getText());
			} else {
				throw new JDOMException("Tag <" + arg.getName() + "> unknown");
			}
		}
	}

	@Override
	public int getType() {
		return TYPE;
	}

	public boolean isEmpty() {
		if (properties.isEmpty()) {
			return true;
		}
		return ((String) properties.get(PKDS)).isEmpty()
				&& ((String) properties.get(PNAME)).isEmpty()
				&& ((String) properties.get(PKDATA)).isEmpty()
				&& ((String) properties.get(PKSOFT)).isEmpty()
				&& ((String) properties.get(PHOST)).isEmpty() && args.isEmpty();
	}

	@Override
	public void setProperty(int row, String property, String value) {
		properties.put(property, value);
		resetStatus();
	}

	public void setArgument(int row, String property, String value) {
		args.put(property, value);
		resetStatus();
	}

	@Override
	public GraphElement getElement() {
		GraphElement nodeE = new GraphElement("node", getIdName());
		nodeE.setAttribute("id", getIdName());
		Element typeE = new Element("data");
		typeE.setAttribute("key", "type");
		typeE.addContent("algorithm");
		nodeE.addContent(typeE);
		if (!isEmpty()) {
			Element descriptionE = new Element("data");
			descriptionE.setAttribute("key", "description");

			Element dmSoftE = new Element("DataMiningSoftware");

			String kdsUrl = (String) properties.get(PKDS);
			if (!kdsUrl.isEmpty() && !kdsUrl.equals(KDSURL_PREFIX)) {
				String title = kdsUrl.substring(kdsUrl.lastIndexOf("/") + 1);
				String name = title;
				int ext = name.lastIndexOf(".");
				if (ext > 0) {
					name = name.substring(0, ext);
				}
				// TODO EPMS shouldn't need the "name" tag if the kds url is set
				dmSoftE.setAttribute("name", name);
				dmSoftE.setAttribute("href", kdsUrl);
				// TODO EPMS shouldn't need the "title" tag if the kdsUrl is set
				dmSoftE.setAttribute("title", title);
			} else {
				String name = (String) properties.get(PNAME);
				if (!name.isEmpty()) {
					dmSoftE.setAttribute("name", name);
				}
			}

			String kData = (String) properties.get(PKDATA);
			if (!kData.isEmpty()) {
				Element descrE = new Element("Description");
				Element kDataE = new Element("KindOfData");
				kDataE.setText(kData);
				descrE.addContent(kDataE);
				dmSoftE.addContent(descrE);
			}

			String kSoft = (String) properties.get(PKSOFT);
			String host = (String) properties.get(PHOST);
			if (!kSoft.isEmpty() || !host.isEmpty() || !args.isEmpty()) {
				Element usageE = new Element("Usage");
				if (!kSoft.isEmpty()) {
					Element kSoftE = new Element("KindOfSoftware");
					kSoftE.setText(kSoft);
					usageE.addContent(kSoftE);
				}
				if (!host.isEmpty()) {
					Element hostE = new Element("HostName");
					hostE.setText(host);
					usageE.addContent(hostE);
				}
				if (!args.isEmpty()) {
					Element argsE = new Element("Args");
					for (Map.Entry<String, String> arg : args.entrySet()) {
						Element argE = new Element("Arg");
						argE.setAttribute("name", arg.getKey());
						argE.setText(arg.getValue());
						argsE.addContent(argE);
					}
					usageE.addContent(argsE);
				}
				dmSoftE.addContent(usageE);
			}

			descriptionE.addContent(dmSoftE);
			nodeE.addContent(descriptionE);
		}
		addPosition(nodeE);
		return nodeE;
	}

	@Override
	public void resetStatus() {
		if (isEmpty()) {
			setStatus(NOINFO);
		} else {
			String kdsUrl = (String) properties.get(PKDS);
			if (kdsUrl.isEmpty() || kdsUrl.equals(KDSURL_PREFIX)) {
				setStatus(SEARCHP_SET);
			} else {
				setStatus(KDSURL_SET);
			}
		}
		if (((String) properties.get(PKDS)).isEmpty()) {
			matchedKDSURL = null;
		}
	}

	public void addArgument(String argName) throws Exception {
		if (args.containsKey(argName)) {
			throw new Exception("Argument already exists.");
		} else {
			args.put(argName, "");
		}
	}

	public void delArgument(String key) {
		args.remove(key);
	}
}
